
package art;

import java.awt.Color;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.Timer;
public class Character2D implements Serializable, FrameByFrameAnimation, ActionListener {

	
	private Shape[] character; 
	private Hashtable<String, Vector<Vector<Shape>>> anim = new Hashtable<String, Vector<Vector<Shape>>>();
	private Hashtable<String, Vector<Vector<Boolean>>> animF = new Hashtable<String,Vector<Vector<Boolean>>>();
	private Hashtable<String, Vector<Vector<BasicStroke>>> animBS = new Hashtable<String, Vector<Vector<BasicStroke>>>();
	private Hashtable<String,Vector<Vector<Color>>> animC = new Hashtable<String,Vector<Vector<Color>>>();
	private Hashtable<String, double[]> transform = new Hashtable<String, double[]>();
	private String names[];
	int frameRate = 20;
	Timer time = new Timer(1000/frameRate, this);
	Drawable canvas;
	
	public Character2D( Hashtable<String, Vector<Vector<Shape>>> anim, Hashtable<String, Vector<Vector<Boolean>>> animF,Hashtable<String, Vector<Vector<BasicStroke>>> animBS, Hashtable<String,Vector<Vector<Color>>> animC, String names[],
			Hashtable<String, double[]> transform){
		this.anim = anim;
		this.animF = animF;
		this.animBS = animBS;
		this.animC = animC;
		this.names = names;
		this.transform = transform;
	}
	
	
	private Queue<String> play = new Queue<String>();
	private Queue<double[]> tran = new Queue<double[]>();

	/**This method must be called, or your Character
	 * object will throw an null exception if it is played!
	 * This method will set the current class which will 
	 * be called every time a sequence's frame changes.
	 * @param g: Must implement Drawable, and also it
	 * must be a Swing or AWT Canvas, which uses paint() or
	 * paintComponent()/paintChilder()/paintBorder().
	 */
	public void setGraphics(Drawable parent){
		canvas = parent;
		
		for(int x=0; x<names.length; x++){
		for(int y=0; y < animBS.get(names[x]).size(); y++){
		for(int z=0;z < animBS.get(names[x]).get(y).size();z++){
			animBS.get(names[x]).get(y).set(z,animBS.get(names[x]).get(y).get(z).refresh());
		}
		}
		}
		
	}
	
	@Override
	public void play(String motion, boolean useTransform){
		play.enqueue(motion);
		
		if(!useTransform){
	tran.enqueue(new double[0]);
	
		}
		else {
			double[] t = transform.get(motion).clone();
			
			for(int x=0; x<7; x++){
				if(x!=1 && x!=2)
					t[x]/=anim.get(motion).size();
				}
			
			tran.enqueue(t);
		}
		time.start();
		}

	@Override
	public void playOne(String motion, boolean useTransform) {
		
		play =  new Queue<String>();
		tran = new Queue<double[]>();
		
		play.enqueue(motion);
		
		if(!useTransform){
			tran.enqueue(new double[0]);
				}
				else {
					double[] t = transform.get(motion).clone();
					
					for(int x=0; x<7; x++){
						if(x!=1 && x!=2)
							t[x]/=anim.get(motion).size();
						}
					
					tran.enqueue(t);
				}
		time.start();
	}

	@Override
	public void setFrameRate(int rate) {
		frameRate=1000/rate;
		time.setDelay(frameRate);
	}

	@Override
	public String[] getAnimationNames() {
		
		return names.clone();
	}

	@Override
	public Hashtable<String, Integer> getLengths() {
		Hashtable<String, Integer> t = new Hashtable<String, Integer>(); 
		
		for(int x=0;x<names.length;x++){
			t.put(names[x],anim.get(names[x]).size());
		}
		
		return t;
	}

	public void play(String motion){
		play.enqueue(motion);
		tran.enqueue(new double[0]);
		time.start();
	}
	
	
	@Override
	public void play(String motion, double[] transforms){
		play.enqueue(motion);
		
		for(int x=0; x<7; x++){
		if(x!=1 && x!=2)
			transforms[x]/=anim.get(motion).size();
		}
		
		tran.enqueue(transforms);
		time.start();
	}

	@Override
	public void stop() {
		time.stop();
		play =  new Queue<String>();
		tran = new Queue<double[]>();
	}

	int second =0;
	
	@Override
	public void actionPerformed(ActionEvent e) {
		
	if (second >= anim.get(play.peek()).size()){
		play.dequeue();
		tran.dequeue();
		second = 0;
		if(play.isEmpty()){
		time.stop();
		canvas.sequenceFinished();
	}}
		
		if(!play.isEmpty()){
			if(tran.peek().length == 0){
			
				character = new Shape[anim.get(play.peek()).get(second).size()];
				
				for(int x=0;x<anim.get(play.peek()).get(second).size(); x++){
				character[x] = anim.get(play.peek()).get(second).get(x);
			}
				
				Color color[] = new Color[anim.get(play.peek()).get(second).size()];
				BasicStroke stroke[] = new BasicStroke[anim.get(play.peek()).get(second).size()];
				boolean fill[] = new boolean[anim.get(play.peek()).get(second).size()];
				
				for(int x=0;x<animC.get(play.peek()).get(second).size(); x++){
					color[x] = animC.get(play.peek()).get(second).get(x);
					stroke[x] = animBS.get(play.peek()).get(second).get(x);
					fill[x] = animF.get(play.peek()).get(second).get(x);
				}
				
				canvas.draw(character, fill, color, stroke);
				second++;
				
				
			} else if(tran.peek().length == 7){
				
				AffineTransform a = new AffineTransform(); 
				AffineTransform b = new AffineTransform(); 
				AffineTransform c = new AffineTransform(); 
				
				if(art.Math.abs(tran.peek()[1])>.001 && art.Math.abs(tran.peek()[2])>.001 && art.Math.abs(tran.peek()[0])>.001){
					if(second>0)tran.peek()[0] += tran.peek()[0]/second;	
				a.setToRotation(tran.peek()[0], tran.peek()[1], tran.peek()[2]);
				}
				
				if(art.Math.abs(tran.peek()[3])>.001 || art.Math.abs(tran.peek()[4])>.001){
				
					if(second>0){
					tran.peek()[3] += tran.peek()[3]/second;	
				tran.peek()[4] += tran.peek()[4]/second;}
				
				b.setToTranslation(tran.peek()[3], tran.peek()[4]);
				}
				
				 if(art.Math.abs(tran.peek()[5])>.001 || art.Math.abs(tran.peek()[6])>.001){
				
					if(second>0){
						tran.peek()[5] += tran.peek()[5]/second;	
					tran.peek()[6] += tran.peek()[6]/second;}
				    
					c.setToScale(tran.peek()[5]+1, tran.peek()[6]+1);
				}
				
				
				
				
				
				character = new Shape[anim.get(play.peek()).get(second).size()];
				
				for(int x=0;x<anim.get(play.peek()).get(second).size(); x++){
				character[x] = a.createTransformedShape(anim.get(play.peek()).get(second).get(x));
				character[x] = b.createTransformedShape(character[x]);
				character[x] = c.createTransformedShape(character[x]);
			}
				
				Color color[] = new Color[anim.get(play.peek()).get(second).size()];
				BasicStroke stroke[] = new BasicStroke[anim.get(play.peek()).get(second).size()];
				boolean fill[] = new boolean[anim.get(play.peek()).get(second).size()];
				
				for(int x=0;x<animC.get(play.peek()).get(second).size(); x++){
					color[x] = animC.get(play.peek()).get(second).get(x);
					stroke[x] = animBS.get(play.peek()).get(second).get(x);
					fill[x] = animF.get(play.peek()).get(second).get(x);
				}
				
				canvas.draw(character, fill, color, stroke);
				second++;
				
				
				
			}
			
		}
		
	}

	@Override
	public void setSequenceTransform(String anim, double[] transform) {
		for(int x = 0; x<7;x++){
			if(x!=1 && x!=2)
			transform[x]/=this.anim.get(anim).size();
		}
		if(!this.transform.contains(transform))
		this.transform.put(anim, transform);
		else this.transform.replace(anim, transform);
	}
	
	

}
